/*!
  * @license MIT (https://github.com/geotrev/snabbdom-transform-jsx-props/blob/master/LICENSE)
  * snabbdom-transform-jsx-props v0.3.0 (https://github.com/geotrev/snabbdom-transform-jsx-props#readme)
  * Copyright 2024 George Treviranus <geowtrev@gmail.com> (https://geotrev.com/)
  */
const kebabToCamel = value => value && value.split("-").map((word, i) => i > 0 ? word[0].toUpperCase() + word.slice(1).toLowerCase() : word.toLowerCase()).join("");

const MODULE_PROPS = {
  hook: "hook",
  on: "on",
  attrs: "attrs",
  props: "props",
  style: "style",
  class: "class",
  dataset: "dataset"
};
const PROP_ALIAS_MAP = {
  className: "className",
  "class-name": "className",
  "tab-index": "tabIndex",
  tabIndex: "tabIndex",
  id: "id"
};
const ATTR = "attr";
const PROP = "prop";
const DATA = "data";
const KEY = "key";
const HYPHEN_CHAR = "-";

const MODULE_NAMES = [...Object.values(MODULE_PROPS), KEY];
function setPropToModule(data, module, key, value) {
  if (data[module]) {
    data[module][key] = value;
  } else {
    data[module] = {
      [key]: value
    };
  }
}
const transform = (oldVNode, _vnode) => {
  const vnode = _vnode || oldVNode;
  if (vnode.data) {
    const nextModuleState = {},
      propKeys = [];
    for (const key in vnode.data) {
      if (MODULE_NAMES.indexOf(key) === -1) {
        propKeys.push(key);
      } else {
        nextModuleState[key] = vnode.data[key];
      }
    }
    const propsLength = propKeys.length;
    let idx = -1;
    while (++idx < propsLength) {
      const propKey = propKeys[idx];
      const propValue = vnode.data[propKey];
      const aliasedKey = PROP_ALIAS_MAP[propKey];
      if (aliasedKey) {
        setPropToModule(nextModuleState, MODULE_PROPS.props, aliasedKey, propValue);
        continue;
      }
      const hyphenIdx = propKey.indexOf(HYPHEN_CHAR);
      if (hyphenIdx > 0) {
        const prefix = propKey.slice(0, hyphenIdx);
        const modKey = prefix === DATA ? DATA : MODULE_PROPS[prefix];
        if (modKey) {
          const postfix = propKey.slice(hyphenIdx + 1);
          if (modKey === DATA) {
            setPropToModule(nextModuleState, MODULE_PROPS.dataset, kebabToCamel(postfix), propValue);
          } else {
            setPropToModule(nextModuleState, modKey, postfix, propValue);
          }
          continue;
        }
        const targetedMod = prefix === ATTR ? MODULE_PROPS.attrs : prefix === PROP ? MODULE_PROPS.props : null;
        if (targetedMod) {
          setPropToModule(nextModuleState, targetedMod, propKey.slice(hyphenIdx + 1), propValue);
          continue;
        }
      }
      setPropToModule(nextModuleState, MODULE_PROPS.attrs, propKey, propValue);
    }
    vnode.data = nextModuleState;
  }
};
const jsxDomPropsModule = {
  create: transform,
  update: transform
};

export { jsxDomPropsModule };
//# sourceMappingURL=snabbdom-transform-jsx-props.js.map
